% This file compares the experimental PTOSw set-point
% tracking results to three things:
% (1). The simulated time-optimal solution
% (2). The simulated PTOSw with CT controller and direct measurement of
% states
% (3). The simulated PTOSw with DT controller and estimated measurement of
% the states. 
%
% Currently, I do not plot the simulation with DT observer. Though
% different, I do not think it shows a large enough difference to warrent
% plotting and just makes the plots too busy. 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get data from experiment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all;
clear all;
clc;
saveon = 0;
addpath('data')
addpath('functions')
addpath('models')
figpath   = getfigpath();
fig1_name = 'brake10.pdf';
fig1_path = fullfile(figpath, fig1_name);
fig2_name = 'brake11.pdf';
fig2_path = fullfile(figpath, fig2_name);


%New scheme with ubp, ubm, and umax
load('ptosParamsRef_march2017.mat', 'ptosParams')

% Color scheme.
fnameColors = 'CS.mat';
load(fnameColors, 'CS');

% fname ='x1r_p5c.csv';
fname='x1r_p5c_jan8_2016v2.csv';

expDatapath  = fullfile(fname);
data1=csvread(expDatapath, 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read data into convinient variables
% N_k = length(y);
N_k = 124;

y  = data1(1:N_k,3);
Ts = ptosParams.Ts;
dt = Ts/30;



vhat = data1(1:N_k,2);
yhat = data1(1:N_k,1);


u = data1(1:N_k,4);

t = Ts*[0:1:N_k-1]';
testOn = 0;
if testOn
    figure(1)
    plot(t, u)
    title('control')

    figure(2)
    subplot(2,1,1), hold on, grid on;
    plot(t, y, 'b', t, yhat, '--k')
    title('y')

    subplot(2,1,2), hold on
    plot(t, vhat, 'b')
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% setup simulations for comparison
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% pull out continuous time matrices
A = ptosParams.A;
B = ptosParams.B;
C = ptosParams.C;

% alpha = ptosParams.alpha;
c      = ptosParams.c;
umax   = ptosParams.umax;
w      = ptosParams.w;
bo     = ptosParams.bo;
x1r    = ptosParams.x1r;
k2     = ptosParams.k2;
ubp   = ptosParams.ubp;
ubm   = ptosParams.ubm;
lambda = ptosParams.lambda;
alpha  = ptosParams.alpha;


xell_plus = ptosParams.xell_plus;
xell_min  = ptosParams.xell_min;
k1        = ptosParams.k1 ;
k2        = ptosParams.k2;
M         = k1/k2;

x1Tol = 0.0001*c;
ptosParams.x1Tol = x1Tol;
% x1Tol = ptosParams.x1Tol;
x2Tol = ptosParams.x2Tol;


L = ptosParams.L;

% pull out discrete time matrices
Az = ptosParams.Az;
Bz = ptosParams.Bz;
Cz = ptosParams.Cz;
L  = ptosParams.L;

% set initial condition
X0   = [y(1), vhat(1)]';

trun = N_k*Ts;
% run the simulation
TDelay = 4*Ts;
Ndelay = 0;
% sim('LRC_linear_discreteObs');

sim('ptos_discreteObsRef_new');
sim('TO_FLAT');
sim('ptos_flexFLAT')
ks = crossing(uopt.Data, uopt.Time);
ks(1:3);
uopt.Data(ks(2):end) = uopt.Data(ks(2):end)*0 + x1r/c;



x1Tol = 0.005*c;
x2Tol = .005*(bo/w);

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot Data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%setup figure
F1 = figure(1);clf

width = 3.5;
height = 2.5;

set(F1, 'color', 'w',...
    'defaulttextinterpreter', 'latex',...
    'PaperUnits','inches',...
    'PaperSize',[width height],...
    'PaperPosition',[0,0,width,height],...
    'Units','inches',...
    'Position',[-10,1,width,height]);


ht = .33;
ax1 = axes('Position', [.12,.58, 1-.12, ht]);
set(ax1, 'FontSize', 8);
ax2 = axes('Position', [.12,.145,1-.12, ht]);
set(ax2, 'FontSize', 8);
% box('on')
fs1 = 10;
fs2 = 8;
fs_leg = 7;
fs_lab = 12;
hold on;
lw = 1.; % standard linewidth


% plot x1 = y
ye = y-x1r;
tend = 0.00065;

% PLOT X1 TIME SERIES
F1.CurrentAxes = ax1;
% subplot(2,1,1), 
hold on, grid on
    p_x1sim = plot(xplant_noobs.Time*1000, xplant_noobs.Data(:,1), 'LineWidth', 1);    
    p_x1exp = plot(t*1000, y, 'LineWidth', lw );
    p_x1ref = plot([t(1), t(end)]*1000, [x1r, x1r], '-.k');
    
    p_x1TO  = plot(xopt.Time(1:75:end)*1000, xopt.Data(1:75:end,1));

    leg1 = legend([p_x1exp, p_x1sim, p_x1TO, p_x1ref], {'PTOS$\omega$, experiment',...
                  'PTOS$\omega$, simulation','Time-Optimal',...
                  '$x_{1r}$'},...
                  'interpreter', 'latex', 'FontSize', fs_leg);
   
    leg1.Position = [0.6211 0.6497 0.4029 0.2627];
    leg1.Box = 'off';
    ylabel('$x_1$~(v)', 'interpreter', 'latex', 'FontSize', fs_lab);
    s = sprintf('$x_{1r} = %.2fcu^+$, $\\lambda = %.2f$, $\\alpha = %.2f$', x1r/c, lambda, alpha);
    title(s, 'interpreter', 'latex', 'FontSize', fs1);
    xlim([0, tend*1000])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
k_ucros = crossing(uopt.Data);
k_ucros = k_ucros(1:2);
uoptInds = [ floor(linspace(1,k_ucros(1),6)), floor(linspace(k_ucros(1)+1,k_ucros(2),8)),...
           floor(linspace(k_ucros(2)+1,length(uopt.data), 35)) ];
size(uoptInds)
% plot control history
F1.CurrentAxes = ax2;
% subplot(2,1,2), 
hold on, grid on;
    p_uexp = plot(t*1000, u, 'LineWidth', lw );
    p_uTO = plot(uopt.Time(uoptInds)*1000, uopt.Data(uoptInds));
    p_usim = plot(uptos_noobs.Time*1000, uptos_noobs.Data, 'LineWidth', 1);

    ylim([-1.1, 1.1]);
    xlim([0, tend*1000])
    xlabel('time (ms)')
    ylabel('$u$~(v)', 'interpreter', 'latex', 'FontSize', fs_lab)


%%%%%%%%%%%%%%%%%
% colors and linestyles

expC = 'k';
expLS = '--';
expLW = 2;
simC  = 'k';
simLS = '-';

toC   = [0.5, 0.5, 0.5];
toLS  = ':';

p_x1exp.Color     = expC;
p_x1exp.LineStyle = expLS;
p_x1exp.LineWidth = expLW;

p_x1sim.Color     = CS.ptosSimC;
p_x1sim.LineStyle = '-';

p_x1sim.LineWidth = 1.3;
p_x1to.Color      = CS.toSimC;
p_x1to.LineStyle  = CS.toSimS;
p_x1TO.LineStyle  = ':';


p_x1TO.Color      = toC; 
p_x1TO.LineWidth  = 1.5;


p_uexp.Color     = expC;
p_uexp.LineStyle = expLS;

p_usim.Color     = simC;
p_usim.LineStyle = simLS;

p_uTO.LineStyle  = toLS;
p_uTO.Color      = toC;
p_uTO.LineWidth  = 1.5;


if 0
    TOL = 0.01;
    ts_opt  = settle_time(xopt.Time, xopt.Data(:,1), x1r, abs(x1r*TOL))
    ts_ptos = settle_time(xplant_noobs.Time, xplant_noobs.Data(:,1), x1r, abs(x1r*TOL))
    ts_exp  = settle_time(t, y, x1r, abs(x1r*TOL))
end

if saveon
    print(gcf, '-dpdf', fig1_path, '-r900', '-opengl'); 
end
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct PTOS plot, and phase plane trajectory
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


lw = 1.3;
F2=figure(2); clf
width = 3.25;
height = 2.5;

set(F2, 'color', 'w',...
    'defaulttextinterpreter', 'latex',...
    'PaperUnits','inches',...
    'PaperSize',[width height],...
    'PaperPosition',[0,0,width,height],...
    'Units','inches',...
    'Position',[-10,1,width,height]);


ht = .75;
ax1 = axes('Position', [.18,.16, 1-.18, ht]);
hold on
set(ax1, 'FontSize', 8);
x1 = 2*alpha*c*ubm:.001:2*alpha*c*ubp;
for i =1:length(x1)
    fp(i) = fptosref(x1(i), ptosParams);
end
%
if 0
    figure(3)
    plot(x1, fp);, grid on, hold on;
    plot(x1, fp-ubp/k2, x1, fp+abs(ubm)/k2);
    plot(xhat.Data(:,1)-x1r, xhat.Data(:,2));
    plot(y-x1r, vhat);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

x1_ptos = x1;
fptos   = fp;

%%%%%%%%%%%%%%%%%%%%%%%%
% plot ptos curve and boundaries
fp_lower = fptos-ubp/k2;
fp_upper = fptos+abs(ubm)/k2;
plotshaded(x1_ptos, [fp_lower; fp_upper], 'k', .1);

%%%%%%%%%%%%%%%%%%%%%%%%
% plot trajectories
p_simphase   = plot(xplant_noobs.Data(:,1)-x1r, xplant_noobs.Data(:,2), 'LineWidth', lw);
p_expphase   = plot(y-x1r, vhat, ':k', 'LineWidth', lw);

kind = 367;
opt_inds = [1:50:kind,kind, kind+50:50:length(xopt.Data(:,1))];
p_optphase   = plot(xopt.Data(opt_inds,1)-x1r, xopt.Data(opt_inds,2));
 p_optphase_hid   = plot(xopt.Data(:,1)-x1r, xopt.Data(:,2), ':k');

ylm = ylim;

%%%%%%%%%%%%%%%%%%%%%%%
% plot xell and c vertical lines
ypts = linspace(ylm(1), ylm(2), 12);
xell_plus_pts = 0*ypts + xell_plus;
xell_min_pts  = 0*ypts - xell_min;
cubp_pts      = 0*ypts + c*ubp;
cubm_pts      = 0*ypts + c*ubm;
p_xell = plot(xell_plus_pts, ypts, xell_min_pts, ypts);
p_c    = plot(cubp_pts, ypts, cubm_pts, ypts);


p_simphase.DisplayName = 'PTOS$\omega$, simulation';
% 'simulation';
p_simphase.Color       = simC;
p_simphase.LineStyle   = simLS;

p_expphase.DisplayName = 'PTOS$\omega$, experiment';
% 'experiment';
p_expphase.Color       = expC;
p_expphase.LineStyle   = expLS;
p_expphase.LineWidth   = expLW;


p_optphase.DisplayName  = 'Time-Optimal';
p_optphase.Color        = toC;
p_optphase.LineStyle    = toLS;
% p_optphase.Marker       = 'x';
p_optphase.LineWidth    = lw;
%%%%%%%%%%%%%%
% fptos and boundares


%%%%%%%%%%%%%%%
% xell and c
p_xell(1).Color     = [.5, .5, .5];
p_xell(1).LineStyle = '-';
p_xell(1).Marker    = '.';

p_xell(2).Color     = [.5, .5, .5];
p_xell(2).LineStyle = '-';
p_xell(2).Marker    = '.';

p_c(1).Color        = [.5, .5, .5];
p_c(2).Color        = [.5, .5, .5];
p_c(1).LineStyle    = '--';
p_c(2).LineStyle    = '--';
p_c(1).Marker    = '.';
p_c(2).Marker    = '.';
p_c(1).MarkerSize = 7;
p_c(2).MarkerSize = 7;
%%%%%%%%%%%%%%%%%%%%%%%%%
% create LABELS and TITLE
title(s, 'interpreter', 'latex', 'FontSize', fs1);
xlim([1.5*ubm*c 1.2*c*ubp]);
xlabel('$x_{1e}$~(v)', 'interpreter', 'latex', 'FontSize', fs_lab);
hylab = ylabel('$x_{2e}$~(v/s)', 'interpreter', 'latex', 'FontSize', fs_lab);
set(hylab, 'Units', 'normalized', 'Position', [-.16, .5, -1]);
%%%%%%%%%%%%%%%%%%%%%%%%%
% write text

h_plus=text(0.22, 0.2, '$\mathcal{U}^{+}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', fs1, 'Units', 'Normalized');
h_min=text(.80,.50, '$\mathcal{U}^{-}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', fs1, 'Units', 'Normalized');
h_Bm=text(.24,.8, '$\mathcal{B}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', fs1, 'Units', 'Normalized');
h_Bp=text(.54,.3, '$\mathcal{B}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', fs1, 'Units', 'Normalized');


h_xln=text(.275,.04, '$-x^-_{\ell}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', fs2, 'Units', 'Normalized');
h_xlp=text(.4,.04, '$x^+_{\ell}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', fs2, 'Units', 'Normalized');
h_xln=text(.10,.04, '$c\bar u^-$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', fs2, 'Units', 'Normalized');
h_xlp=text(.96,.04, '$c\bar u^+$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', fs2, 'Units', 'Normalized');

%%%%%%%%%%%%%%%%%%%%%%%%
% create legend
leg2 = legend([p_expphase, p_simphase, p_optphase]);
leg2.Interpreter = 'latex';
leg2.Position = [0.5648 0.7277 0.4336 0.1814];
leg2.FontSize = fs_leg;
grid on
leg2.Box = 'on';
set(gca, 'Box', 'on')
if saveon
    print(gcf, '-dpdf', fig2_path, '-r900', '-opengl'); 
end


%% Now, save data from the experiment to G_data.txt. This is read by the latex root file.

% First, calculate some relevant parameters.
clc
G = tf(ss(ptosParams.A, ptosParams.B, ptosParams.C, 0));
sysCL = ss(ptosParams.A-ptosParams.B*[k1, k2], ptosParams.B, ptosParams.C, 0);
w_cl = sqrt(-sysCL.a(2,1));
f_cl = w_cl/2/pi;
z_cl = -sysCL.a(2,2)/w_cl/2;
[num, den] = tfdata(G, 'v');

ww  = sqrt(den(3));
z   = den(2)/2/ww;
TF_str = tf2tex(G);

k2str = strrep(num2str(k2, '%.2e'), '-0', '-');
ts_optstr = strrep(num2str(ts_opt, '%.2e'), '-0', '-');
ts_ptosstr = strrep(num2str(ts_ptos, '%.2e'), '-0', '-');
ts_expstr  = strrep(num2str(ts_exp, '%.2e'), '-0', '-');

L_str = sprintf('\\begin{bmatrix}%g & %g\\end{bmatrix}^T', round(L(1), 3), L(2));
ptosData = struct('lam', round(lambda,4), 'alpha', round(alpha, 2), 'w', w, 'z', round(z,2),...
                  'bo', bo, 'G', TF_str, 'FsKhz', (1/Ts)/1000, 'L', L_str,...
                   'x1rc', x1r/c, 'fclKhz', f_cl/1000, 'zcl', z_cl, 'fsfcl', (1/Ts)/f_cl,...
                   'kone', k1, 'ktwo', k2str, 'tsopt', ts_optstr,...
                   'tsptos', ts_ptosstr, 'tsexp', ts_expstr);
              
char_data = struct2meta(ptosData);

delete(tfPath);
fid = fopen(tfPath, 'w');
fprintf(fid, '%s', char_data);
fclose(fid);








