% This file generates and plots the experimental results for Fig. 12. If
% you set the parameer runExp =1, this script will attempt to run the
% LabView VI which defines the PTOSwz implementation. The resulting data
% will be saved to a .csv file. Otherwise, the data will simply be read
% from that file and the figures ploted.
%
% If the parameter savedata = 1, the results will be saved into a JSON-like
% text file (tex_data_path) which is read by latex.

clear all;
clc;

saveon = 0;
runExp = 0;
savedata = 0;
addpath('data');
addpath('functions');

load('plants_damp.mat', 'plants_damp');
load('CS.mat', 'CS');
tex_data_path = fullfile('latex', 'G_nodamp.txt');
fig_name = 'brake12.pdf';
fig1_path = fullfile(getfigpath(), fig_name);
plants = plants_damp;
% These parameters define paths for the experimental implementation.
controlDataPath = fullfile(pwd, 'data');
controlDataName = 'controlParams.csv';
controlDataFile = fullfile(controlDataPath, controlDataName);
vipath  = fullfile(pwd, 'labview', 'play_LRCptosREF_LUT.vi');


umax = 1;
f = @dyn;
Gss  = plants.Gss;
Gss.iodelay = 0;


ptosParams.TsTicks = 400;
Ts   = ptosParams.TsTicks/40e6; % 10e-06;
ptosParams.Ts = Ts;
Gzss = c2d(Gss, Ts);

% make ss matrices accessible
A = Gss.a;
B = Gss.b;
C = Gss.c;

Az = Gzss.a;
Bz = Gzss.b;
Cz = Gzss.c;

bo = B(2);
w  = sqrt(-A(2,1));
z  = -A(2,2)/w/2;   
wd = w*sqrt(1-z^2);
c  = bo/w^2;


% define the reference
x1r = -0.5*c;
% form umin umax
ubp    = umax - x1r/c;
ubm    = -umax - x1r/c;
alpha  = 0.85;
lambda = 0.15;
x_ell  = lambda*c*alpha;


% setup time vector field
dt = Ts/100;
t = (0:dt:pi/wd/2)';



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GENERATE SWITCHING CURVE IN UNSHIFTED COORDS. UMIN/MAX = +-1;, 

params.A = A;
params.B = B;


t  = flipud(t); % integrate backwards
Xf = [0 0]';


% Nominal scurve
params.u = umax*alpha;
[t, x_p] = ode45(@(t,x)f(t,x,params), t, Xf);
params.u = -umax*alpha;
[t, x_n] = ode45(@(t,x)f(t,x,params), t, Xf);
Scurve = [flipud(x_n); x_p];
Scurve = forceMono(Scurve);
ptosDamp_obj = ptosDamp_nom(Scurve, lambda, c, umax, alpha);
x1_s         = ptosDamp_obj.ptosCurves.x1_s;
x2_ptos      = ptosDamp_obj.ptosCurves.x2_ptos;

% True Scurve
params.u = umax;
[t, x_p] = ode45(@(t,x)f(t,x,params), t, Xf);
params.u = -umax;
[t, x_n] = ode45(@(t,x)f(t,x,params), t, Xf);
Scurve = [flipud(x_n); x_p];
Scurve = forceMono(Scurve);


trun = 2*pi/wd;
uss = 1;
x0 = [uss*c; 0];
x0_obs = x0;
inv(eye(2) - Az)*Bz*1;


sclneg = 1/abs(ubm);
sclpos = 1/abs(ubp);
dt = Ts/30;
sim('dampPtos_scaleState');
sim('dampScaleTO');


ks = crossing(uopt.Data, uopt.Time);
kk = ks(2);
uopt.Data(kk:end) = uopt.Data(kk:end)*0 + x1r/c;


tscl = 1000; 
%%

F1 = figure(1);clf

width = 3.5;
height = 2.8;

set(F1, 'color', 'w',...
    'defaulttextinterpreter', 'latex',...
    'PaperUnits','inches',...
    'PaperSize',[width height],...
    'PaperPosition',[0,0,width,height],...
    'Units','inches',...
    'Position',[-10,1,width,height]);


fs1 = 10;
fs2 = 8;
fs_lab = 12;
fs_leg = 7;


lft = .16;
ht = .23;
marg = .03;
marg2 = .09;
top1 = 1-marg-ht;
ax1 = axes('Position', [lft, top1, 1-lft-.025, ht]);
set(gca, 'FontSize', 7.5)
top2 = top1 - marg2 - ht;
ax2 = axes('Position', [lft, top2, 1-lft-.025, ht]);
set(gca, 'FontSize', 7.5)
ylim([-10000, 5000])


top3 = top2 - marg2 - ht+.011;
ax3 = axes('Position', [lft, top3, 1-lft-.025, ht]);
set(gca, 'FontSize', 7.5)

F1.CurrentAxes = ax1;
hold on

    p_x1sim_ptos =   plot(x_ptos.Time*tscl, x_ptos.Data(:,1));
    p_x1sim_ptos.DisplayName = 'PTOS$\omega\zeta$ (simulation)';
    
    p_x1sim_opt  = plot(xopt.Time*tscl, xopt.Data(:,1));
    p_x1sim_opt.DisplayName = 'Time Optimal';

F1.CurrentAxes = ax2;
hold on
    p_x2sim_ptos = plot(x_ptos.Time*tscl, x_ptos.Data(:,2), 'k');
    p_x2sim_opt  = plot(xopt.Time*tscl, xopt.Data(:,2));

F1.CurrentAxes = ax3;
hold on
    p_usim_ptos = plot(u_ptos.Time*tscl, u_ptos.Data, 'k');
    p_usim_ptos.DisplayName = 'PTOS$\omega\zeta$ (simulation)';
        
    p_usim_opt  = plot(uopt.Time*tscl, uopt.Data);
    p_usim_opt.DisplayName = 'Time Optimal';

% SETUP EXPERIMENT
% Design Estimator gain:
M            = 4.5;
g            = tf([3*bo], [1 2*.95*M*w (M*w)^2]);
gz           = c2d(g, Ts);
p            = pole(gz);
Lz = acker(Az', Cz', p)';


x1s_lut  = linspace(-2*c, 2*c, 1024);
fptosLUT = interp1(x1_s, x2_ptos, x1s_lut);


paramData = [c; ubp; ubm; alpha; umax; ptosDamp_obj.k2; x1r; x1r/c];
if runExp
    saveControlData_lutptos(fptosLUT', Az, Bz, Lz, paramData, controlDataFile);
    TsTicks = 40e6*Ts;
    [e, vi] = setupVI(vipath, 'Ts(Ticks)', TsTicks);
    vi.Run;
end


[~, den] = tfdata(plants.Gs, 'v');
ww = sqrt(den(3));
z = den(2)/ww/2;

sysCL = ss(A - B*[ptosDamp_obj.k1, ptosDamp_obj.k2], B, [1 0], 0);
f_cl = sqrt(-sysCL.a(2,1))/2/pi;
f_ol = ww/2/pi;

% Load experimental data.
expData = csvread(fullfile('data', 'expData01.csv'));

y_exp = expData(:,1);
u_exp = expData(:,2);
x1hatExp = expData(:,3);
x2hatExp = expData(:,4);
x1hatExp = [x0_obs(1); x1hatExp(1:end-1)];
x2hatExp = [x0_obs(2); x2hatExp(1:end-1)];


nn = length(y_exp);
t_exp = [0:1:nn-1]'*Ts;


tmax = 0.6;

F1.CurrentAxes = ax1;
    stit = sprintf('$x_{1r} = %.2fc$, $\\lambda = %.2f$, $\\alpha = %.2f$, $\\zeta = %.3f$',...
            x1r/c, lambda, alpha, z);

          text(.5, 1.0323, stit, 'Units', 'normalized', 'FontSize', fs2,...
              'HorizontalAlignment', 'center')
   p_x1exp =  plot(t_exp*tscl, y_exp, '--m');
   p_x1exp.DisplayName = 'PTOS$\omega\zeta$ (experiment)';
   p_x1ref = plot([t(1), t(end)]*1000, [x1r, x1r], '-.k');
   p_x1ref.DisplayName = '$x_{1r}$';
   
   xlim([0, tmax])
   ylim([-1, 1.1])
   ylabel('$x_1$~(v)', 'interpreter', 'latex', 'FontSize', fs_lab);
       
    grid on
    
F1.CurrentAxes = ax2;
    p_x1exp = plot(t_exp*tscl, x2hatExp, '--m');
    xlim([0, tmax])
    ylab2 = ylabel('$x_2, \hat x_2$~(v/s)', 'interpreter', 'latex', 'FontSize', fs_lab);
    set(ylab2, 'units', 'normalized', 'Position', [-.13 .5, -1]);
    grid on

F1.CurrentAxes = ax3;
    p_uexp = plot(t_exp*tscl, u_exp, '--m');
    p_uexp.DisplayName = 'PTOS$\omega\zeta$ (experiment)';
  hleg1 =   legend([p_uexp, p_usim_ptos, p_usim_opt, p_x1ref]);
  hleg1.Interpreter = 'latex';
  hleg1.FontSize = 7;
  hleg1.Position = [0.1727 0.1426 0.3952 0.1866];
  hleg1.Box = 'off';
    grid on
    hxlab = xlabel('time (ms)', 'FontSize', 10);
    set(hxlab, 'Units', 'Normalized', 'Position', [.5, -.25, -1])
    ylabel('$u$~(v)', 'interpreter', 'latex', 'FontSize', fs_lab)
    xlim([0, tmax])
    ylim([-1.1, 1.1]);

% Setup line colors and styles
p_x1exp.Color     = CS.expC;
p_x1exp.LineStyle = CS.expS;

p_x1sim_ptos.Color     = CS.ptosSimC;
p_x1sim_ptos.LineStyle = CS.ptosSimS;

p_x1sim_opt.Color      = CS.toSimC;
p_x1sim_opt.LineStyle  = CS.toSimS;
p_x1sim_opt.LineWidth  = CS.toSimlw;

p_x2exp.Color     = CS.expC;
p_x2exp.LineStyle = CS.expS;

p_x2sim_ptos.Color     = CS.ptosSimC;
p_x2sim_ptos.LineStyle = CS.ptosSimS;

p_x2sim_opt.Color      = CS.toSimC;
p_x2sim_opt.LineStyle  = CS.toSimS;
p_x2sim_opt.LineWidth  = CS.toSimlw;

p_usim_opt.Color      = CS.toSimC;
p_usim_opt.LineStyle  = CS.toSimS;
p_usim_opt.LineWidth  = CS.toSimlw;

TOL = 0.01;
ts_opt  = settle_time(xopt.Time, xopt.Data(:,1), x1r, abs(x1r*TOL));
ts_ptos = settle_time(x_ptos.Time, x_ptos.Data(:,1), x1r, abs(x1r*TOL));
ts_exp  = settle_time(t_exp, y_exp, x1r, abs(x1r*TOL));
   
   
if saveon
    print(F1, '-dpdf', fig1_path, '-r900', '-painters'); 
end
   


k1 = ptosDamp_obj.k1;
k2 = ptosDamp_obj.k2;

G     = tf(ss(A, B, C, 0));
sysCL = ss(A-B*[k1, k2], B, C, 0);
w_cl  = sqrt(-sysCL.a(2,1));
f_cl  = w_cl/2/pi;
z_cl  = -sysCL.a(2,2)/w_cl/2;
[num, den] = tfdata(G, 'v');

ww  = sqrt(den(3));
z   = den(2)/2/ww;

L_str = sprintf('\\begin{bmatrix}%g & %g\\end{bmatrix}^T', round(Lz(1), 3), round(Lz(2),0));

TF_str = tf2tex(G);
k2str = strrep(num2str(k2, '%.2e'), '-0', '-');
ts_optstr = strrep(num2str(ts_opt, '%.2e'), '-0', '-');
ts_ptosstr = strrep(num2str(ts_ptos, '%.2e'), '-0', '-');
ts_expstr  = strrep(num2str(ts_exp, '%.2e'), '-0', '-');


ptosData = struct('lam', round(lambda,2), 'xell', x_ell, 'alpha', round(alpha, 2), 'w', w, 'z', round(z,2),...
                    'bo', bo, 'G', TF_str, 'FsKhz', (1/Ts)/1000, 'L', L_str,...
                   'x1rc', x1r/c, 'fclKhz', f_cl/1000, 'zcl', z_cl, 'fsfcl', (1/Ts)/f_cl,...
                  'kone', k1, 'ktwo', k2str, 'tsopt', ts_optstr,...
                   'tsptos', ts_ptosstr, 'tsexp', ts_expstr);

char_data = struct2meta(ptosData);

if savedata
    delete(tex_data_path)
    fid = fopen(tex_data_path, 'w');
    fprintf(fid, '%c', char_data);

    fclose(fid);
end

%%    
ptosExp.y_exp = y_exp;
ptosExp.u_exp = u_exp;
ptosExp.x1hatExp = x1hatExp;
ptosExp.x2hatExp = x2hatExp;

% ptosExp.Xhatsim = Xhat;
ptosExp.x_ptos = x_ptos;
ptosExp.params = paramData;
ptosExp.Gss = Gss;
ptosExp.Gzss = Gzss;
ptosExp.Scurve = Scurve;
ptosExp.ptosDamp = ptosDamp_obj;
ptosExp.x1r = x1r;

if savedata
    save('data\07-18-2016_ptosExp_damp.mat', 'ptosExp');
end











