clear, clc

F30 = figure(30); clf
width = 3.5;
height = 2.25;

set(F30, 'color', 'w',...
    'defaulttextinterpreter', 'latex',...
    'PaperUnits','inches',...
    'PaperSize',[width height],...
    'PaperPosition',[0,0,width,height],...
    'Units','inches',...
    'Position',[-10,1,width,height]);

% for PDF version:
% ht = .855
% ax1 = axes('Position', [.18,.13, 1-.18, ht]);

% For eps version
ht = .84
ax1 = axes('Position', [.18,.14, 1-.18, ht]);

set(gca, 'FontSize', 7.5)
% box('on')
fs1 = 10;
fs2 = 8;
fs_lab = 12;
fs_leg = 7;


addpath('functions')
% close all;
highbw = 0;

% Setup line colors.
optC = ':k';
linC = '-.r';
ptosC='k';

umax = 1;
b1 = 0;
bo = 4.47806e7;

w = sqrt(4.44851e7);
zeta = 843.519/w/2;

A = [0  1;
    -w^2 0];
B = [0;
    bo];
u = 1;
wd = w*sqrt(1 - zeta^2);

A_plant = A; B_plant = B;
% A_plant(2,2) = -2*w*zeta;

c = bo/w^2;

x0    = -[-.5*umax*c, 0]';
% x0    = [umax*c, 0]';
dt    = 1e-7;
trun  = 1*2*pi/w;
trun = floor(trun/dt)*dt;
x1Tol = (.0000001*c);
x2Tol = (.000001*bo/w);

tlim  = 0.5*2*pi/w;
% ==============================
% Ptos Parameters
lambda = 0.1; 
alpha  = .85;
k1   = (1-lambda)/(alpha*c*lambda);
k2   = sqrt(2*lambda - lambda^2)/(w*c*lambda*alpha);
M    = k1/k2;

x1r_s = -[-.3, 0.1, .6]*c*umax;
% x1r_s = [2]*c;
slin = []; sopt = []; sptos = [];

for iter = 1:length(x1r_s)

% Design the PTOS
x1r = x1r_s(iter);
ubp   = umax - x1r/c;
ubm   =  -umax  - x1r/c;


xell_plus = lambda*alpha*c*ubp;
xell_min  = lambda*alpha*c*ubm;

% Closed loop natural frequency.
w_hat = w*sqrt(1 + (1-lambda)/(alpha*lambda));
eigs = eig(A-B*[k1, k2]);
w_hat_num = abs(eigs(1));
fprintf('CL omega (closed form) %f\nCL omega (eigen) %f\n',...
        w_hat, w_hat_num);

params = struct('A', A, 'B', B, 'ubp', ubp, 'ubm', ubm);
quiet = 1;
Scurve = sCurve(@dyn, params, w, dt, quiet);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now, run simulate the time optimal solution first.
% This provides an easy way to get the point where the trajectory crosses
% the the time-optimal switching curve.
sim('timeOptFlexDamp')
k_s = crossing(u_opt.Data);

x1c = xout_opt(k_s(1), 1) - x1r;
x2c = xout_opt(k_s(1), 2);
rho = abs(x2c/x1c);
if highbw
    k = 2*umax*(w_hat^2 - w^2)/(bo*rho);
else
    k = (w_hat^2 - w^2)/(bo*rho);
end

% Calculate the closed loop bandwidth.
% Control is -k*[rho*x1 + x2]
%            = [k*rho, k][x1, x2]
eigs = eig(A-B*[k*rho, k]);
w_hat_num_highgain = max(abs(eigs));
fprintf('CL omega (high gain) %f\n', w_hat_num_highgain);



x0_plant = x0;
sim('equiv_S_line_PROX2');

sim('ptos_flexFLAT_compare')
if 0
figure(0+iter); clf
% subplot(2,1,1)
    plot(Scurve(:,1), Scurve(:,2))
    hold on, grid on;
    plot(Xopt_scurve.Data(:,1)-x1r, Xopt_scurve.Data(:,2), optC )
    plot(xout_ptos(:,1)-x1r, xout_ptos(:,2), ptosC)
    plot(x_highgain.Data(:,1)-x1r, x_highgain.Data(:,2), linC)
    xlim([-2*c*umax, 2*c*umax]);
    plot(x1c, x2c, 'xk')
    dx = abs(c)/500;
    xx1 = c*ubm:dx:c*ubp;
    sline = -xx1*rho;
    plot(xx1, sline, 'm');
    plot(xx1, -rho*xx1-(1/k)*ubp, 'm')
    plot(xx1, -rho*xx1-(1/k)*ubm, 'm')
    
end

t0 = 0;
t1 = trun;



F30 = figure(30); 
if iter == 0 
    clf; 
end
    hold on, grid on
%     y0 = x0(1);
%     y_normal = abs(x1r - y0);
%     t_norm = 1e3;
%     y_hg_norm = (x_highgain.Data(:,1) - y0)/y_normal;
%     y_ptos_norm = (xout_ptos(:,1) - y0)/y_normal;
%     y_opt_norm = (Xopt_scurve.Data(:,1) - y0)/y_normal;
    y0 = x0(1);
    y_normal = x1r - x0(1); %abs(x1r - y0);
    t_norm = 1e3;
    y_hg_norm = (x_highgain.Data(:,1) - x1r)/y_normal;
    y_ptos_norm = (xout_ptos(:,1) - x1r)/y_normal;
    y_opt_norm = (Xopt_scurve.Data(:, 1) - x1r)/y_normal;

    p_hg_y = plot(x_highgain.Time*t_norm, y_hg_norm, linC, 'LineWidth', 1.1);
    p_ptos_y = plot(uptos_noobs.Time*t_norm, y_ptos_norm, ptosC, 'LineWidth', 1.5);
    p_opt_y = plot(Xopt_scurve.Time*t_norm, y_opt_norm, optC, 'LineWidth', 1.5);
    xlim([0, tlim]*t_norm)
    hxlab = xlabel('time (ms)', 'interpreter', 'latex', 'FontSize', fs_lab)
    %for PDF
    set(hxlab, 'Units', 'normalized', 'Position', [.5, -.07, -1])
    % for EPS
    set(hxlab, 'Units', 'normalized', 'Position', [.5, -.09, -1])
    
    hylab = ylabel('$\frac{x_{1e}(t)}{-x_{1e}(0)}$',...
        'interpreter', 'latex', 'FontSize', 14);
    
    
    h_leg = legend([p_hg_y, p_ptos_y, p_opt_y], 'high-gain linear',  'PTOS$\omega$',...
          'time-optimal');
    h_leg.Location =   'SouthEast';
    h_leg.Interpreter = 'latex';
    h_leg.FontSize = fs_leg;
    h_leg.Box = 'off';


% y0 = x0(1);
peakVal = abs(x0(1) - x1r);
ST=0.01;
uppSet = [x1r+ST*peakVal, x1r+ST*peakVal] - x1r;
lowSet = [x1r-ST*peakVal, x1r-ST*peakVal] - x1r;
yfinal = 0;


slin = [slin; stepinfo(x_highgain.Data(:,1)-x1r, x_highgain.Time, yfinal, ...
               'SettlingTimeThreshold',ST)];
sptos = [sptos; stepinfo(xout_ptos(:,1)-x1r, uptos_noobs.Time,yfinal,...
                 'SettlingTimeThreshold',ST)];
sopt = [sopt; stepinfo(Xopt_scurve.Data(:,1)-x1r, Xopt_scurve.Time, yfinal,...
               'SettlingTimeThreshold',ST)];
           
fprintf('===============================\n')
fprintf('Time-Opt. settle time %f\n', sopt(iter).SettlingTime);
fprintf('HG settle time %f\n', slin(iter).SettlingTime);
fprintf('PTOS settle time %f\n', sptos(iter).SettlingTime);

fprintf('settle-HG/settle-ptos %f\n', slin(iter).SettlingTime/sptos(iter).SettlingTime);
fprintf('===============================\n')
end % end main iteration over x1rs.

print(F30, '-dpdf', 'latex_zip/figures-noembed/brake13.pdf', '-r900', '-painters'); 
%%
saveEps(F30, 'latex/figures/brake13.eps')

