clear;
clc
close all
addpath('functions')
addpath('models')
figpath   = getfigpath();
fig_name = 'brake04.pdf';
fig_path = fullfile(figpath, fig_name); 
saveon = 0;


% define plot styles:
s_opt = '--b';
s_ptos = 'g';
s_opt_pert = '-.r';
s_fp = ':k';
s_fto = 'k';
step_sz=.0000006;
bo = 1.8276*10^7;
w=2*pi*3.175*10^2;
c=bo/w^2;
zeta=0;

%make the state space matrices
A=[0 1;-(w)^2 0;]; 
B=[0; bo];
C=[1 0];
D=[0; 0];
sys = ss(A, B, C, 0);


x1_s = [-2*c:.001:2*c]';
x1_opt = x1_s;
x2_opt = -sign(x1_s).*w.*sqrt(2*c*abs(x1_s) - x1_s.^2);

% PTOSw control parameters
umax = 1;
x_ell = 0.1;
alpha = 0.85;
k1 = (alpha*c - x_ell)/(alpha*c*x_ell);
k2 = sqrt(2*alpha*c*x_ell - x_ell^2)/(w*alpha*c*x_ell);
Mmin = k1/k2;

[x1_ptos, fptos] = ptosCurves_local(c, w, bo, alpha, x_ell, k1, k2);
fp_upper = fptos + 1/k2;
fp_lower = fptos - 1/k2;


% simulation parameters
trun = 1.5*pi/w;
dt = 1/100e3;
trun =ceil(trun/dt)*dt;

Xo = [-.8*c; 0];
X0 = Xo;

% simulate erturbed system
bo_pert = 1.05;

B = B*bo_pert;
x1_tol = 0.0000001;
x2_tol = 0.0000002;

sim('fig4_timeOptFlex')
u_pert = u_opt;
x_pert = x_opt;

x1_tol = 0.01;
x2_tol = 0.02;

sim('fig4_sim_ptos');

B = sys.b;
sim('fig4_timeOptFlex') %ideal trajectory
tend = uptos.Time(end);
u_opt =timeseries([u_opt.Data; [u_opt.Time(end):dt:tend]'*0],...
                  [u_opt.Time; [u_opt.Time(end):dt:tend]']);

% ----------------------------------------------------------------------- %
%                             Plot, format axes etc.                      %
% ----------------------------------------------------------------------- %
F1 = figure(1);clf; % Create Figure
width = 3.5;
height = 2.15;
set(F1, 'color', 'w',...
    'defaulttextinterpreter', 'latex',...
    'PaperUnits','inches',...
    'PaperSize',[width height],...
    'PaperPosition',[0,0,width,height],...
    'Units','inches',...
    'Position',[1,1,width,height]);

ht = .815;
ax1 = axes('Position', [.075,.12,.54, ht]);
ax2 = axes('Position', [.740, .12,.31, ht]);

F1.CurrentAxes = ax1;
box('on'); % Define box around whole figure
hold on;

plotshaded(x1_ptos, [fp_lower; fp_upper], [.85, .85 .85], 1);

h_scurve    = plot(x1_opt, x2_opt, s_fto); %switch curve
h_fp       = plot(x1_ptos, fptos, s_fp);

plot(xout_opt(1,1), xout_opt(1,2), '.k', 'MarkerSize', 25)
plot_xopt1 = plot(xout_opt(:,1), xout_opt(:,2), s_opt);
plot_xptos = plot(xptos.Data(:,1), xptos.Data(:,2), s_ptos);
h_opt_pert = plot(x_pert.Data(:,1), x_pert.Data(:,2), s_opt_pert);

set(plot_xopt1,'LineWidth',1.5);  %ideal trajectory
set(h_scurve,'LineWidth',1);

fs1 = 10;

% --------------------- build the axes -----------------------------------%

xlim([-1.5*c, .05*c])
ylim([-1000, 16000]) 

set(ax1, 'XTick', [-2*c:c:2*c]);
set(ax1, 'YTick', [0]);
set(ax1, 'YTickLabel', [])

yticks=get(ax1, 'ytick');
xticks=get(ax1, 'xtick');

set(ax1, 'XTickLabel', [])

my_yticks={'$0$' };
my_xticks={'$-2cu^+$', '$c\bar{u}^-$','$0$','$cu^+$','$2cu^+$'};
axs=axis;
HorizontalOffset=0-.1;
VertOffset=-1000;

for i=1:length(yticks)
  text(axs(1)+HorizontalOffset, yticks(i),my_yticks(i),'HorizontalAlignment',...
      'Right','interpreter', 'latex', 'FontSize', fs1);   

end

for i=2:3
  text(xticks(i)+0, axs(3)+VertOffset, my_xticks(i),'HorizontalAlignment',...
      'Center','interpreter', 'latex', 'FontSize',fs1);   

end

text(.15, .14, '$\mathcal{U}^+$', 'units', 'normalized', 'interpreter', 'latex',...
    'FontSize', fs1)
text(.85, .5, '$\mathcal{U}^-$', 'units', 'normalized', 'interpreter', 'latex',...
    'FontSize', fs1)
grid on

% Create xlabel
hxlab=xlabel('$x_{1e}$','FontSize',12,'FontName','Times New Roman', 'rot', 0);
set(hxlab, 'Units', 'normalized', 'Position', [.5, -.04, -1]);

% Create ylabel
hylab=ylabel('$x_{2e}$','FontSize',12,'FontName','Times New Roman','rot',90);
set(hylab, 'Units', 'Normalized', 'Position', [-.04, 0.45, 0]);


hleg1=legend([h_scurve, h_fp, plot_xopt1, h_opt_pert, plot_xptos], '$f_{to}$',...
             '$f_p$', '$x_e(t)$ \& $\bar{u}(t)$, opt. (ideal)',...
             '$x_e(t)$ \& $\bar{u}(t)$, opt. (perturbed)',...
             '$x_e(t)$ \& $\bar{u}(t)$, PTOS$\omega$ (perturbed)');
         
hleg1.Position =  [0.0474 0.6014 0.5754 0.3388];

set(hleg1,  'FontSize', 7);
set(hleg1,'Interpreter','latex')
set(hleg1, 'Box', 'off')         
         
stit = sprintf('$x_{\\ell} = %.2f$, $\\alpha = %.2f$', x_ell, alpha);
text(.25, 1.041, stit, 'interpreter', 'latex', 'FontSize', fs1,...
    'Units', 'normalized')

F1.CurrentAxes = ax2;
box('on')
set(ax2, 'FontSize', fs1)
hold on;
h_opt_pert = plot(u_pert.Time*1000, u_pert.Data, s_opt_pert);
h_uopt     = plot(uptos.Time*1000, uptos.Data, s_ptos,  'LineWidth', 1.5);
h_u1ptos   = plot(u_opt.Time*1000, u_opt.Data, s_opt,  'LineWidth', 1.5);


hxlab=xlabel('time', 'FontSize', fs1);
hylab=ylabel('$\bar{u}(t)$', 'FontSize', fs1, 'FontName','Times New Roman',...
    'FontAngle','italic','rot',90);

hylab.Position = [-0.3, 0.0815 -1];
hxlab.Position = [0.9500 -1.38 -1.0000];
ylim([-1.2, 1.2])
xlim([0, 1.9])
grid on

axis_positon = get(gca, 'Position');
ax1 = gca;


yticks=get(ax1, 'ytick');
my_yticks={'$\bar{u}^-$', '$0$', '$\bar{u}^+$'};
set(ax1, 'YTickLabel', [])
axs = axis;
HorizontalOffset=0;
VertOffset=-1000;
text(axs(1)+HorizontalOffset, yticks(1),my_yticks(1),'HorizontalAlignment',...
    'Right','interpreter', 'latex', 'FontSize', fs1);   
text(axs(1)+HorizontalOffset-.1, yticks(3),my_yticks(2),'HorizontalAlignment',...
    'Right','interpreter', 'latex', 'FontSize', fs1);   
text(axs(1)+HorizontalOffset, yticks(end),my_yticks(3),'HorizontalAlignment',...
    'Right','interpreter', 'latex', 'FontSize', fs1);   

% set(gca, 'Position', axis_positon);
xticks=get(ax2, 'xtick');

set(ax1, 'XTickLabel', [])

my_xticks={'$0$', '1'};
text(.0, -.05, '0', 'Units', 'normalized')
text(.5, -.05, '1', 'Units', 'normalized')

if saveon
    print(F1, '-dpdf', fig_path, '-r600');
end



