clear;
clc;

addpath('functions');
saveon = 0;
figpath   = getfigpath();
fig_name = 'brake05.pdf';
fig_path = fullfile(figpath, fig_name); 



bo = 1.8276*10^7;
w = 2*pi*3.175*10^2;
zeta = 0.2;
sigma = w*zeta;
wd = w*sqrt(1-zeta^2);
c  = bo/w^2;

psi = 1+exp(sigma*pi/wd);


umax = 5;
alpha = 0.85;

xell    =.6*alpha*c*umax;
% hang on to these to test against the known, undamped case.
% k1    = (alpha*c*umax-xell)/(alpha*c*xell)
% k2    = sqrt(2*c*alpha*umax*xell - xell^2)/(alpha*w*c*xell)
fto_xell = fxk_newton(w, zeta, bo, sign(xell)*umax, alpha, xell);
k1overk2 = -(-w^2*xell - 2*sigma*fto_xell + bo*alpha*umax)/fto_xell;
umaxoverk2 = -fto_xell - k1overk2*xell;
k2 = umax/umaxoverk2;
k1 = k1overk2*k2;


x1  = [-psi*alpha*c*umax:.01:psi*c*alpha*umax];
xlv =-xell:.1:xell;

x2upper_lin =-(k1/k2)*xlv-umax/k2;
x2lower_lin =-(k1/k2)*xlv-umax/k2;
ftl         =-(k1/k2)*xlv;


fto = x1*0;
for k=1:length(x1)
    xk = x1(k);
    if xk ~= 0
        [fxk, t_k] = fxk_newton(w, zeta, bo, sign(xk)*umax, alpha, xk);
        fto(k) = fxk;
    end
end

kxl_plus    = find(x1 >=xell, 1, 'first');
kxl_min     = find(x1<=-xell, 1, 'last');

fptos_plus  = umax/k2 + fto(kxl_plus:end);
fptos_min   = -umax/k2 + fto(1:kxl_min);

% =========================================================================
% BEGIN PLOTTING
% =========================================================================

% Setup Figure
F1     = figure(1); clf
width = 3.5;
height = 2.125;

set(F1, 'color', 'w',...
    'defaulttextinterpreter', 'latex',...
    'PaperUnits','inches',...
    'PaperSize',[width height],...
    'PaperPosition',[0,0,width,height],...
    'Units','inches',...
    'Position',[1,1,width,height]);


ht = .79;
ax1 = axes('Position', [.1,.13,1-.135, ht]);
box('on')
fs1 = 10;
fs2 = 8;
fs_lab = 12;
hold on;
p2 = plot([x1(1:kxl_min)], [fptos_min], ':k', 'linewidth', 1.5);
p2 = plot([x1(kxl_plus:end), psi*c*umax*alpha], [fptos_plus, umax/k2], ':k', 'linewidth', 1.5);

p4 = plot(xlv, ftl, '-.k', 'linewidth', 1.5); % (k1/k2)x1
p1 = plot(xlv, x2upper_lin, '--k','linewidth', 1.5); %1/k2 + (k1/k2)x1


p3 = plot([x1, psi*alpha*c*umax], [fto, 0], '-k', 'linewidth', 1.); % f(x1), old time optimal curve
p55 = plot([-psi*c*umax, -psi*alpha*c*umax], [0, 0], '-k',... 
           [psi*alpha*c*umax, psi*c*umax], [0, 0], '-k', 'LineWidth', 1.);
grid on


ax=gca;
xticks = [-psi*c*umax, -psi*alpha*c*umax, -c*umax, 0, c*umax, psi*alpha*c*umax, psi*c*umax];
set(ax, 'XTick', xticks);
set(ax, 'YTick', [0]);
set(ax, 'YTickLabel', [])

xlim([-psi*c*umax, psi*c*umax]);
ylim([-1.5*bo*umax/w, 1.5*bo*umax/w]);


yticks=get(ax, 'ytick');
set(ax, 'XTickLabel', [])

my_yticks={'$\frac{-b_ou^+}{\omega}$','$0$','$\frac{b_ou^+}{\omega}$'};
my_xticks={'$-\psi cu^+$', '$-\psi \alpha cu^+$', '$-cu^+$', '$0$','$cu^+$',...
              '$\psi \alpha cu^+$', '$\psi cu^+$'};
axs=axis;
HorizontalOffset=-2;
VertOffset=6000;


% single y-tick
set(gca, 'TickDir', 'out')
text(axs(1)+HorizontalOffset, yticks(1),'$0$','HorizontalAlignment',...
    'Right','interpreter', 'latex', 'FontSize', fs1);   
%             psi psi.alp    cu   0    cu,   psi.alp,  psi
horzoffsets = [-4.5,      4.5,   -2,  0,   3      -4,         5];
horzoffsets = [-3.5,      4.5,   -2,  0,   3      -4,         3];
for i=1:length(xticks)
    
    horzoffset = horzoffsets(i);
    text(xticks(i)+horzoffset, axs(3)-VertOffset, my_xticks(i),'HorizontalAlignment',...
      'Center','interpreter', 'latex', 'FontSize',fs2);   

end
hleg1=legend([p1, p2, p3, p4],'$-\frac{u^+}{k_2}-\frac{k_1}{k_2}x_1$',...
       '$f_{to}(x_1)\pm\frac{u^+}{k_2}$','$f_{to}(x_1)$','$-\frac{k_1}{k_2}x_1$');
%
set(hleg1,  'FontSize', 9);
set(hleg1,'Interpreter','latex');
hleg1.Position= [0.5921 0.5832 0.3320 0.3402];
set(hleg1,'box','on')

% Create xlabel
hxlab=xlabel('$x_1$','FontSize',fs_lab,'FontName','Times New Roman','FontAngle','italic');

% Create ylabel
hylab=ylabel('$x_2$','FontSize',fs_lab,'FontName','Times New Roman','FontAngle','italic','rot',90);


set(hylab, 'Units', 'Normalized', 'Position', [-.05, 0.5, 0]);
set(hxlab, 'Units', 'Normalized', 'Position', [.5, -.05, 0]);
set(hxlab, 'interpreter', 'latex');
set(hylab, 'interpreter', 'latex');


% plot vertical xell and alpha.c.umax lines.
ylm       = ylim;
ypts = linspace(ylm(1), ylm(2), 8);
xnegalpc_pts = ypts*0-alpha*c*umax;
p13       = plot(xnegalpc_pts, ypts, '--.k', 'MarkerSize',7.5);

ypts(end) = [];
xell_pts = 0*ypts + xell;

p10=plot(xell_pts, ypts, '-k.', 'LineWidth', 1, 'MarkerSize', 7.5);

xalpc_pts = ypts*0+alpha*c*umax;
p12       = plot(xalpc_pts, ypts, '--.k', 'MarkerSize',7.5);

text(.544, .035, '$x_{\ell}$', 'Units', 'Normalized', 'interpreter', 'latex', 'FontSize', fs2)
text(.35, .035, '$-\alpha cu^+$', 'Units', 'Normalized', 'interpreter', 'latex', 'FontSize', fs2)
text(.655, .035, '$\alpha cu^+$', 'Units', 'Normalized', 'interpreter', 'latex', 'FontSize', fs2)
text(.05, .85, '$\mathcal{U}^-$', 'Units', 'Normalized', 'interpreter', 'latex', 'FontSize', fs2)
text(.1, .15, '$\mathcal{U}^+$', 'Units', 'Normalized', 'interpreter', 'latex', 'FontSize', fs2)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot size markes

[~, kc_min] = max(fto);
ftonc  = fto(kc_min);
fpnc   = fto(kc_min)-umax/k2;
ac = -x1(kc_min);

text(.19, .82, '$\frac{u^+}{k_2}$', 'Units', 'Normalized', 'interpreter', 'latex', 'FontSize', 12)

% Convert to normalized coords.
[x1fp, x2f] = ds2nfu([-ac, -ac], [ftonc,fpnc]);
annotation('doublearrow', x1fp, x2f, 'headstyle', 'vback3');

s = sprintf('$\\alpha = %.2f$', alpha);
title(s, 'interpreter', 'latex', 'FontSize', fs2)


if saveon
    print(gcf, '-dpdf', fig_path, '-r600', '-painters'); 
end





