clear all;
close all;
clc;
addpath('functions');
addpath('models')
saveon = 0;

load('CS.mat', 'CS')
step_sz = 1e-6;
dt      = step_sz;



%============================================
% Model Definition
bo   = 1.8276*10^7;
w    = 2*pi*3.175*10^2;
c    = bo/w^2;
zeta = 0;


%make the state space matrices
A    = [0 1;-(w)^2 0;]; 
B    = [0; bo];
C    = [1 0];
D    = [0; 0];
trun = 2*pi/w;
trun = ceil(trun/dt)*dt;

sys  = ss(A, B, C, 0);

% Define the control
umax     = 1; %saturator limit
alpha  = 0.75; %discount factor
lambda = 0.1;

x1r    = -0.5*c;

ubp = umax - x1r/c;
ubm = -umax - x1r/c;

% ====================================
% Build up PTOS Switchiig curves and boundaries

xell_plus = lambda*alpha*c*ubp;
xell_min  = lambda*alpha*c*ubm;

k2        = sqrt(2*lambda - lambda^2)/(w*alpha*c*lambda);
k1        = (1 - lambda)/(alpha*c*lambda);
M         = k1/k2;


ptosParams.xell_min = xell_min;
ptosParams.xell_plus = xell_plus;
ptosParams.k2        = k2;
ptosParams.k1        = k1;;
ptosParams.alpha     = alpha;
ptosParams.lambda    = lambda;
ptosParams.M         = M;
ptosParams.ubm      = ubm;
ptosParams.ubp      = ubp;
ptosParams.w         = w;
ptosParams.c         = c;
ptosParams.x1r       = x1r;

% ====================
% stopping parameters
x1Tol = 2.6*10^-12;
x2Tol = .00001;


dx       = 0.001;

x1n      = (2*alpha*c*ubm:dx:-xell_min)';
x1p      = (xell_plus:dx:2*alpha*c*ubp)';
x_ells   = (xell_min+dx:dx:xell_plus-dx)';


x1       = [x1n; x_ells; x1p];

% FTO_disc = @(x, alp) -sign(x).* w.*sqrt(alp*2*c*abs(x) - x.^2);

for i = 1:length(x1)
    Fptos(i) = fptosref(x1(i), ptosParams);
end
x2_upper = Fptos - ubm/k2;
x2_lower = Fptos - ubp/k2;



x0=[-9*c+x1r, 0];
Xo = x0;
% =====================================
% Simulate PTOSw

% x0_s = 2.1*alpha*c;
% x0_s = alpha*2*c*1.5;
% x0 = [x0_s 0];
trun = abs((pi/w)*x0(1));
trun = ceil(trun/dt)*dt;
sim('ptos_flexFLAT_chop');
  
ks = crossing(xplant.Data(:,2));
kk1 = ks(2);


xptos_example = xplant; 
% ===========================
% Simulate zero input response
trun = pi/w;
trun = ceil(trun/dt)*dt;
u = 0;
x0 = [x0(1)-x1r, x0(2)];
sim('rotate');
x_rot_u0_1 = x_rotate;


u=0;
x0 = [xplant.Data(kk1,1)-x1r, 0]';
sim('rotate');
x_rot_u0_2 = x_rotate;




if 0
    % to see what happens if alpha <0.5
    ks = crossing(uptos.Data);

    T0 = (0:step_sz:2*pi/w)';
    [yy1, t1, xx1] = lsim(sys, -1+0*T0, T0, x0);
    x0 = xptos_example.Data(ks(1), :);
    [yy2, t, xx2,] =lsim(sys, 1+0*T0, T0, x0);
end


% ----------------------------------------------------------------------- %
%                      Figure setup, format axes etc.                     %
% ----------------------------------------------------------------------- %
F1    = figure(1);clf

width = 3.5;
height = 3.5;

set(F1, 'color', 'w',...
    'defaulttextinterpreter', 'latex',...
    'PaperUnits','inches',...
    'PaperSize',[width height],...
    'PaperPosition',[0,0,width,height],...
    'Units','inches',...
    'Position',[1,1,width,height]),...
    

ht = .85;
ax1 = axes('Position', [.15,.091,1-.15, ht]);

box('on')
fs1 = 10;
fs2 = 8;
fs_lab = 12;
hold on;

% ===================
% Plot scurve, and ptos.
% [fptosC, x2_upper, x2_lower] = chopCurves(Fptos, x2_upper, x2_lower, ptosParams, x1);
fptosC = Fptos;


plotshaded(x1', [x2_lower; x2_upper], [.85, .85 .85], 1);

% ====================
% Plot trajectories.
htraj1 = plot(xptos_example.Data(:,1)-x1r, xptos_example.Data(:,2), '--k'); % fptos + straigt line response
htraj2 = plot(x_rot_u0_2.Data(1:50:end,1), x_rot_u0_2.Data(1:50:end,2), '-.k'); %u = -1 input response
htraj3 = plot(x_rot_u0_1.Data(1:50:end,1), x_rot_u0_1.Data(1:50:end,2), '-.k');   %zero input response

%===================
% Plot Dots
dotbig = 10;
plot(x_rot_u0_1.Data(1,1), x_rot_u0_1.Data(1,2), '.k', 'MarkerSize', dotbig);
plot(xptos_example.Data(1,1)-x1r, xptos_example.Data(1,2), '.k', 'MarkerSize', dotbig);


%=================
% Plot distance marks
xsings = crossing(xptos_example.Data(:,2));
k1     = xsings(2);
k2     = xsings(3);

y1 = 1.5*bo/w;
xx1 = xptos_example.Data(k1,1)-x1r;
xx2 = xptos_example.Data(k2,1)-x1r;
xx3 = x_rot_u0_1.Data(end,1);

archStyle = '-';
archColor = 'k';
td = 0.1;
plot([xx1, xx1], [0, -y1], archColor)
plot([xx3, xx3], [0, -y1], archColor)
plot([xx1-td*c, xx1+td*c],  [0-td*bo/w, 0+td*bo/w], archColor)
plot([xx3-td*c, xx3+td*c],  [0-td*bo/w, 0+td*bo/w], archColor)

plot([xx1-td*c, xx3+td*c], [-y1+td*bo/w, -y1+td*bo/w], archColor)
plot([xx1-td*c, xx1+td*c],  [-y1+td*bo/w-td*bo/w, -y1+td*bo/w+td*bo/w], archColor)
plot([xx3-td*c, xx3+td*c],  [-y1+td*bo/w-td*bo/w, -y1+td*bo/w+td*bo/w], archColor)


plot([xx2, xx2], [0, -y1], archColor)


plot([-xx1, -xx1], [0, -y1], archColor)
plot([-xx1-td*c, -xx1+td*c],  [0-td*bo/w, 0+td*bo/w], archColor)
plot([xx2-td*c, xx2+td*c],  [0-td*bo/w, 0+td*bo/w], archColor)

plot([xx2+td*c, -xx1-td*c], [-y1+td*bo/w, -y1+td*bo/w], archColor)

plot([xx2-td*c, xx2+td*c],  [-y1+td*bo/w-td*bo/w, -y1+td*bo/w+td*bo/w], archColor)
plot([-xx1-td*c, -xx1+td*c],  [-y1+td*bo/w-td*bo/w, -y1+td*bo/w+td*bo/w], archColor)



% =================
% Plot horizontal switching curve.
if exist('sy', 'var')
    delete(sy);
    delete(sx);
end
xlm = xlim;
hhorzn = plot([xlm(1) 2*alpha*c*ubm], [0, 0], CS.fpC,...
              [2*alpha*c*ubp xlm(2)], [0, 0], CS.fpC);

% =========================================================================
% Now, build up the axes. 

ax=gca;

% xlim([1.2*min(min(xn_s(:, kx1s))),  1.2*max(max(xp_s(:, kx1s)))]);

xlm = xlim;
Nx1   = ceil(xlm(1)/c);
Nx2   = floor(xlm(2)/c);
nxdiff = 4;
xdiff = nxdiff*c;


xtick_pos = [0:xdiff:Nx2*xdiff/nxdiff];
xtick_neg = fliplr([0:-xdiff:Nx1*xdiff/nxdiff]);
xtick_neg = xtick_neg(1:end-1);
% set(ax, 'XTick', [Nx1*xdiff/nxdiff:xdiff:Nx2*xdiff/nxdiff]);
set(ax, 'XTick', [xtick_neg, xtick_pos]);

set(ax, 'XTickLabel', [])


ylm = ylim;
ylim([1.1*ylm(1), 1.3*ylm(2)]);
ylm = ylm;
Ny1    = ceil(ylm(1)/((bo)/w));
Ny2    = floor(ylm(2)/((bo)/w));
nydiff = 2;
ydiff  = nydiff*bo/w;

% ylim([1.2*min(min(xp_s)), 1.4*max([max(xn_s(:,end))])])
set(ax, 'YTick', [Ny1*ydiff/nydiff:ydiff:Ny2*ydiff/nydiff]);
set(ax, 'YTickLabel', [])


yticks=get(ax, 'ytick');
xticks=get(ax, 'xtick');

%==========
% Build strings for custom labels.

j = 1;
%Negative x1
for iter = 0:-nxdiff:Nx1
    if iter ==0
        my_xticks_neg{j} = sprintf('$0$');
    else
        my_xticks_neg{j} = sprintf('$%d cu^+$', iter);
    end
j = j+1;
end

% Positive x1
j = 1;
my_xticks_neg = fliplr(my_xticks_neg(2:end));

for iter = 0:nxdiff:Nx2
    if iter ==0
        my_xticks_pos{j} = sprintf('$0$');
    else
        my_xticks_pos{j} = sprintf('$%d cu^+$', iter);
    end
j = j+1;
end
my_xticks = [my_xticks_neg, my_xticks_pos];


j = 1;
for iter = Ny1:nydiff:Ny2
    if iter ==1 
       my_yticks{j} = sprintf('$\\frac{b_ou^+}{\\omega}$'); 
    elseif iter ==-1
        my_yticks{j} = sprintf('$\\frac{ -b_ou^+}{\\omega}$') ;
    elseif iter ==0
        my_yticks{j} = sprintf('$0\\:$');
    else
        my_yticks{j} = sprintf('$\\frac{ %d b_ou^+}{\\omega}$', iter);
    end
j = j+1;
end

xlm = xlim;
ylm = ylim;

axs=axis;
HorizontalOffset=4.1*c;
VertOffset=1.5*1000;


xdist = xlm(2) - xlm(1);
ydist = ylm(2) - ylm(1);

yax = xlm(1);
xax = ylm(1);
for i=1:2:length(my_yticks)
    
    
  sy(i) = text(yax, yticks(i),my_yticks(i),'HorizontalAlignment','Right','interpreter', 'latex',...
      'FontSize', fs1);

end

for i=1:length(my_xticks)
  sx(i) = text(xticks(i), xax(1) - ydist*.01, my_xticks(i),'HorizontalAlignment','Center',...
      'VerticalAlignment', 'Top', 'interpreter', 'latex', 'FontSize',fs2);   

end

grid on

% Create xlabel
hxlab=xlabel('$x_{1e}$','FontSize',fs_lab,'FontName','Times New Roman','FontAngle','italic');

% Create ylabel
hylab=ylabel('$x_{2e}$','FontSize',fs_lab,'FontName','Times New Roman','FontAngle','italic','rot',90);
set(hylab, 'Units', 'Normalized', 'Position', [-0.1, 0.4, 0]);
set(hxlab, 'Units', 'Normalized', 'Position', [.5, -0.045, 0]);
set(hxlab, 'interpreter', 'latex');
set(hylab, 'interpreter', 'latex');


%===================
% Plot vertical lines
ylm = ylim;
hcn = plot([c*ubm, c*ubm], ylm, CS.cC);
hcp = plot([c*ubp, c*ubp], ylm, CS.cC);

hcn.LineStyle = ':';
hcn.Color = 'k';
hcp.LineStyle = ':';
hcp.Color = 'k';

% =====================
% Legend and plot text.
if exist('st', 'var')
    delete(st)
end

st(1) = text(0.1, 0.1, '$\mathcal{U}^+$', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', fs1);
st(2) = text(.9, .8, '$\mathcal{U}^-$', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', fs1);

st(3) = text(0.82, 0.44, 'A', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', fs1);

% st(4) = text(0.89, .55, 'C', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', 12);
st(5) = text(0.82, 0.6, 'B', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', fs1);

st(6) = text(c*ubm*1.1, xax*01.01, '$c\bar{u}^-$', 'VerticalAlignment', 'Bottom', 'HorizontalAlignment', 'right', 'interpreter', 'latex', 'FontSize', fs1);
st(7) = text(c*ubp*1.1, xax*01.01, '$c\bar{u}^+$', 'VerticalAlignment', 'Bottom', 'HorizontalAlignment', 'left', 'interpreter', 'latex', 'FontSize', fs1);

st(8) = text(xx1-.2*c, -1.99*y1, '$|2c\bar{u}^-|$', 'VerticalAlignment', 'Bottom', 'HorizontalAlignment', 'left',...
        'interpreter', 'latex', 'FontSize', fs1);
    
st(8) = text(xx2-2*c, -1.8*y1, '$2c\bar{u}^+$', 'VerticalAlignment', 'Bottom', 'HorizontalAlignment', 'left',...
        'interpreter', 'latex', 'FontSize', fs1);    

%=======================
% Legend stuff

s = sprintf('$\\lambda = %.1f, x_{1r} = %.1f c, \\alpha = %.2f$', lambda, x1r/c, alpha);
title(s, 'interpreter', 'latex')
h=legend([htraj1, htraj3], '(A) $u(t)=\bar{u}_{p}$', '(B) zero input responses');
h.Location='northwest';
h.Position = [0.1494 0.7889 0.5654 0.1527];
h.Interpreter='latex';

if saveon
    fig1_path = 'latex/figures/brake06.pdf';
    print(gcf, '-dpdf', fig1_path, '-r600', '-painters'); 
end






                

