% This script plots the PTOS region with chopped of "tails" for the REAL
% alpha.
clear
clc
close all
saveon = 0;
addpath('functions');
addpath('hatchfillpkg');

bo   = 1.8276*10^7;
w    = 2*pi*3.175*10^2;
c    = bo/w^2;
zeta = 0;

%make the state space matrices
A = [0 1;-(w)^2 0;]; 
B = [0; bo];
C = [1 0];
D = [0; 0];

umax = 1; %saturator limit
x1r = -0.3*c;
ubm = -umax - x1r/c;
ubp = umax  - x1r/c;
alpha = 0.85;
% PTOS PARAMS
lambda = 0.1;
xell_plus = lambda*alpha*c*ubp;
xell_min  = lambda*alpha*c*ubm;




k1   =(1 - lambda)/(alpha*c*lambda);
k2   =sqrt(2*lambda-lambda^2)/(alpha*w*c*lambda);
M = k1/k2;
ptosParams.xell_min = xell_min;
ptosParams.xell_plus = xell_plus;
ptosParams.k2        = k2;
ptosParams.k1        = k1;
ptosParams.alpha     = alpha;
ptosParams.lambda    = lambda;
ptosParams.M         = M;
ptosParams.ubm      = ubm;
ptosParams.ubp      = ubp;
ptosParams.w         = w;
ptosParams.c         = c;
ptosParams.x1r       = x1r;



dx = alpha*c/500;
x1 = (2*alpha*c*ubm:dx:2*alpha*c*ubp);
x1_plus = (xell_plus:dx:2*alpha*c*ubp);
x1_min  = (2*alpha*c*ubm:dx:xell_min);
x1_lin  = (xell_min:dx:xell_plus);
% x1to = (-2*c:dx:2*c);
% Fto = -sign(x1to).*w.*sqrt(2*c*abs(x1to) - x1to.^2);
% Fto(1) = 0; Fto(end) = 0; % Ensure it touches the x1-axis.
% ====================================
% quasi-time optimal flat line 

x1_left  = (-4*c:dx:-2*alpha*c);
x2_left  = 0*x1_left;
x1_right = (2*alpha*c:dx:4*c);
x2_right = 0*x1_right;



for i = 1:length(x1)
    Fptos(i) = fptosref(x1(i), ptosParams);
end
x2_upper = Fptos - ubm/k2;
x2_lower = Fptos - ubp/k2;

Fptos_2 = Fptos;
x2_upper_2 = x2_upper;
x2_lower_2 = x2_lower;

curves_ptosx1 = x1;

% ----------------------------------------------------------------------- %
%                      Figure setup, format axes etc.                     %
% ----------------------------------------------------------------------- %

f1 = figure(1); clf;
width = 6.5;
height = 2.3;

set(f1, 'color', 'w',...
    'defaulttextinterpreter', 'latex',...
    'PaperUnits','inches',...
    'PaperSize',[width height],...
    'PaperPosition',[0,0,width,height],...
    'Units','inches',...
    'Position',[1,1,width,height]);


ht = .83;
ax1 = axes('Position', [.051,.11,1-.06, ht]);
box('on')
fs1 = 10;
fs2 = 8;
fs_lab = 12;
hold on;

% Ensure that curves touch the x1-axis where they should.
curves_optx2(1)=0; curves_optx2(9185)=0; curves_optx2(27553)=0; curves_optx2(36737)=0;


plot_ptoscurve = plot(x1, Fptos_2, 'k');
plot_upper     = plot(x1, x2_upper_2-100, 'k');
plot_lower     = plot(x1, x2_lower_2, 'k');

%=======================
% Plot shaded area
  
    

% For tails
ks_u = crossing(x2_upper);
ks_l = crossing(x2_lower);

k = ks_u(end);


% Get curves that we chop off, to plot shading in B only
[fptosC, x2_upperC, x2_lowerC] = chopCurves(Fptos, x2_upper, x2_lower, ptosParams, x1);


ps2 = plotshaded(x1, [x2_upperC; x2_lowerC], 'k', .45); %shading in region B

ps1 = plotshaded(x1(k:end), [x2_upper(k:end); x2_lower(k:end)*0], 'k', .1); %shading in T^+
ps3 = plotshaded(x1(1:ks_l(1)), [x2_lower(1:ks_l(1)); zeros(1,ks_l(1))], 'k', .1);% shading in T^-

%linear
x_ells = [xell_min:dx:xell_plus];
ps4 = plotshaded(x_ells, [-M*x_ells-ubp/k2; -M*x_ells-ubm/k2], 'k', .6); 
Hlin = hatchfill(ps4, 'single', 30, 5, [0.5 0.5 0.5]);
set(Hlin, 'Color', 'white')


% ===================
% Plot vertical lines
xll  = plot([xell_plus, xell_plus], [-2*bo/w, 2*bo/w], '--g', 'LineWidth',1.1);
xll  = plot([xell_min, xell_min], [-2*bo/w, 2*bo/w], '--g', 'LineWidth',1.1);

xcpl = plot([c*ubm, c*ubm], [-2*bo/w, 2*bo/w], '-.b', 'LineWidth', 1.1);
xcp  = plot([c*ubp, c*ubp], [-2*bo/w, 2*bo/w], '-.b', 'LineWidth', 1.1);


plot_ptoscurve = plot(x1, Fptos_2, 'k');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% Start Formatting the plot                                               %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% ========================================================
% Build the axes
%
ax = gca;

xlim([2.5*c*ubm, 2.3*c*ubp])

xlm = xlim;
ylim([1.2*min(Fptos-ubp/k2), 1.35*max(Fptos-ubm/k2)])
ylm = ylim;
plot_optleft   = plot([xlm(1), 2*alpha*c*ubm], [0, 0], 'k', 'LineWidth', 1.125);
plot_optright  = plot([2*alpha*c*ubp, xlm(2)], [0 0], 'k', 'LineWidth', 1.125);

% More vertical lines, secondary importance
[~, kk1] = max(x2_lower);
[~, kk2] = min(x2_lower);

plot([2*c*alpha*ubm, 2*c*alpha*ubm], [ylm(1), x2_lower(1)], ':k')
plot([c*alpha*ubm, c*alpha*ubm], [ylm(1), x2_lower(kk1)], ':k')

plot([c*alpha*ubp, c*alpha*ubp], [ylm(1), x2_lower(kk2)], ':k')
plot([2*c*alpha*ubp, 2*c*alpha*ubp], [ylm(1), x2_lower(end)], ':k')


xlm    = xlim;
Nx1    = ceil(xlm(1)/c);
Nx2    = floor(xlm(2)/c);
nxdiff = 1;
xdiff  = nxdiff*c;


set(ax, 'XTick', [Nx1*xdiff/nxdiff:xdiff:Nx2*xdiff/nxdiff]);
set(ax, 'XTickLabel', [])

ylm    = ylim;
Ny1    = ceil(ylm(1)/((bo)/w));
Ny2    = floor(ylm(2)/((bo)/w));
nydiff = 1;
ydiff  = nydiff*bo/w;

set(ax, 'YTick', [0]);
set(ax, 'YTickLabel', [])

yticks=get(ax, 'ytick');
xticks=get(ax, 'xtick');

%==========
% Build strings for custom labels.
j = 1;
for iter = Nx1:nxdiff:Nx2
    if iter ==1 
       my_xticks{j} = sprintf('$cu^+$'); 
    elseif iter ==-1
        my_xticks{j} = sprintf('$-cu^+$') ;
    elseif iter ==0
        my_xticks{j} = sprintf('$0$');
    else
        my_xticks{j} = sprintf('$%d cu^+$', iter);
    end
j = j+1;
end




my_yticks = {'0'};


xlm = xlim;
ylm = ylim;

axs =axis;

HorizontalOffset = 4.1*c;
VertOffset       = 1.5*1000;


xdist = xlm(2) - xlm(1);
ydist = ylm(2) - ylm(1);

yax   = xlm(1);
xax   = ylm(1);
for i=1:length(my_yticks)
    
    
  text(yax, yticks(i),my_yticks(i),'HorizontalAlignment','Right','interpreter', 'latex',...
      'FontSize', fs1);

end

for i=1:length(my_xticks)
  s(i) = text(xticks(i), xax(1) - ydist*.01, my_xticks(i),'HorizontalAlignment','Center',...
      'VerticalAlignment', 'Top', 'interpreter', 'latex', 'FontSize',fs1);   

end

grid on


% Create ylabel
hylab=ylabel('$x_{2e}$','FontSize',fs_lab,'FontName','Times New Roman',...
                'FontAngle','italic','rot',90, 'Units', 'Normalized', ...
                'Position', [-0.025, 0.5, 0],'interpreter', 'latex');
% Create xlabel
hxlab=xlabel('$x_{1e}$','FontSize',fs_lab,'FontName','Times New Roman',...
             'FontAngle','italic', 'Units', 'Normalized', ...
             'Position', [0.45, -0.035, 0], 'interpreter', 'latex');

% =============================
% Plot dots
dotthick = 10;
k = find(x2_upper < 0, 1, 'last');
kn = find(x2_lower >0, 1, 'first');

plot(0, -ubm/k2, '.k', 'MarkerSize', dotthick);
plot(0, -ubp/k2, '.k', 'MarkerSize', dotthick);


% Dots for G
plot(2*alpha*c*ubp, 2/k2, '.k', 'MarkerSize', dotthick);
plot(2*alpha*c*ubm, -2/k2, '.k', 'MarkerSize', dotthick);

%

% Build coordinates for dots and ABCDE etc
x_A = 0;
y_A = -ubm/k2;

x_B = xell_plus;
y_B = -xell_plus*k1/k2 - ubm/k2;

x_C = x1(k);
y_C = x2_upper(k);

x_D = x1(end);
y_D = x2_lower(end);

x_E = xell_plus;
y_E = -(k1/k2)*xell_plus - ubp/k2;

x_F = 0;
y_F = -ubp/k2;

x_Bbar = xell_min;
y_Bbar = -(k1/k2)*xell_min -ubp/k2;

x_Cbar = x1(kn);
y_Cbar = x2_lower(kn);

x_Dbar = x1(1);
y_Dbar = x2_upper(1);


x_Ebar = xell_min;
y_Ebar = -(k1/k2)*xell_min -ubm/k2;

plot(x_A, y_A, '.k', 'MarkerSize', dotthick);
plot(x_B,  y_B, '.k', 'MarkerSize', dotthick);

plot(x_C, y_C, '.k',  'MarkerSize', dotthick);
plot(x_D, y_D, '.k',  'MarkerSize', dotthick);

plot(x_E, y_E, '.k',  'MarkerSize', dotthick);
plot(x_F, y_F, '.k',  'MarkerSize', dotthick);


plot(x_Bbar, y_Bbar, '.k',  'MarkerSize', dotthick);
plot(x_Cbar, y_Cbar, '.k',  'MarkerSize', dotthick);
plot(x_Dbar, y_Dbar, '.k',  'MarkerSize', dotthick);

plot(x_Ebar, y_Ebar, '.k',  'MarkerSize', dotthick);

% ==============================
% Plot Text
h_uplus = text(0.2, 0.2, '$\mathcal{U^+}$', 'HorizontalAlignment','Right',...
            'interpreter', 'latex', 'FontSize', fs1, 'Units', 'Normalized');

h_min = text(0.55, 0.85, '$\mathcal{U^-}$', 'HorizontalAlignment','Right',...
          'interpreter', 'latex', 'FontSize', fs1, 'Units', 'Normalized');
h_Breg = text(0.6, 0.18, '$\mathcal{B}$', 'HorizontalAlignment','Right',...
          'interpreter', 'latex', 'FontSize', fs1, 'Units', 'Normalized');

h_Breg2 = text(0.25, 0.79, '$\mathcal{B}$', 'HorizontalAlignment','Right',...
          'interpreter', 'latex', 'FontSize', fs1, 'Units', 'Normalized');

      
h_Tplus = text(0.8, 0.79, '$\mathcal{T}^+$', 'HorizontalAlignment','Right',...
            'interpreter', 'latex', 'FontSize', fs1, 'Units', 'Normalized');
h_Tmin = text(0.1, 0.88, '$\mathcal{T^-}$', 'HorizontalAlignment','Right',...
            'interpreter', 'latex', 'FontSize', fs1, 'Units', 'Normalized');

h_L = text(0.38, 0.32, '$\mathcal{L}$', 'HorizontalAlignment','Right',...
          'interpreter', 'latex', 'FontSize', fs1, 'Units', 'Normalized');
      
h_xlp = text(0.42, 0.04, '$x_{\ell}^+$', 'HorizontalAlignment','Right',...
            'interpreter', 'latex', 'FontSize', fs2, 'Units', 'Normalized');

h_xln = text(0.35, 0.04, '$x_{\ell}^-$', 'HorizontalAlignment','Right',...
            'interpreter', 'latex', 'FontSize', fs2, 'Units', 'Normalized');
            
h_xcp = text(0.68, 0.04, '$c \bar{u}^+$', 'HorizontalAlignment','Right',...
            'interpreter', 'latex', 'FontSize', fs2, 'Units', 'Normalized');
h_xcn = text(0.22, 0.04, '$c \bar{u}^-$', 'HorizontalAlignment','Right',...
            'interpreter', 'latex', 'FontSize', fs2, 'Units', 'Normalized');

% auxilary vertical lines
h_2alpc_ubm    = text(0.115, 0.04, '$2\alpha c\bar{u}^-$',...
                 'HorizontalAlignment','Right','interpreter', 'latex',...
                 'FontSize', 10, 'Units', 'Normalized');

h_alp_c_ubm    = text(0.3, 0.04, '$\alpha c \bar{u}^-$',...
                     'HorizontalAlignment','Right','interpreter',...
                     'latex', 'FontSize', 10, 'Units', 'Normalized');
            
h_alp_c_ubp    = text(0.635, 0.04, '$\alpha c\bar{u}^+$',...
                     'HorizontalAlignment','Right','interpreter',...
                     'latex', 'FontSize', 10, 'Units', 'Normalized');

th_alp_c_ubm    = text(0.9051, 0.04, '$2 \alpha c \bar{u}^+$', 'HorizontalAlignment','Right','interpreter',...
                'latex', 'FontSize', 10, 'Units', 'Normalized');

%draw arrows
ax1 = [.13, .175];
ay1 = [0.85, 0.605];
a1=annotation('arrow', ax1, ay1, 'Units', 'Normalized');
a1.HeadStyle = 'vback3';
a1.HeadLength = 4;
%
ax2 = [.785, .83];
ay2 = [0.76, 0.6798];
a2=annotation('arrow', ax2, ay2, 'Units', 'Normalized');
a2.HeadStyle = 'vback3';
a2.HeadLength = 4;


ax3 = [.422, .422]*.95;
ay3 = [0.42, 0.49];
a3=annotation('arrow', ax3, ay3, 'Units', 'Normalized');
a3.HeadStyle = 'vback3';
a3.HeadLength = 4;

% %%%%%%%%%%%%%%%%%%%%5
yt = bo/w;
xt = xticks(2) - xticks(1);
h_A     = text(x_A+xt*0.01, y_A*1.01, 'A', 'FontSize', fs1, 'interpreter', 'latex');
h_B     = text(x_B, y_B, 'B',          'FontSize', fs1, 'interpreter', 'latex');
h_C     = text(x_C*.96, y_C+yt*0.051, 'C', 'FontSize', fs1, 'interpreter', 'latex');
h_D     = text(x_D, y_D+yt*0.051, 'D', 'FontSize', fs1, 'interpreter', 'latex');
h_E     = text(x_E, y_E, 'E', 'FontSize', fs1, 'interpreter', 'latex');

h_F     = text(x_F+0.1, y_F*.95, '\={A}', 'FontSize', fs1, 'interpreter', 'latex');
h_G     = text(x_D, yt*.35, 'G', 'FontSize', fs1, 'interpreter', 'latex');


% %%%%%%%%%%%%%%%%%%%%%
% Negative, bar
h_Bbar     = text(x_Bbar*2.5, y_Bbar*1.2, '\={B}',          'FontSize', fs1, 'interpreter', 'latex');
h_Cbar     = text(x_Cbar, y_Cbar-yt*0.12, '\={C}',          'FontSize', fs1, 'interpreter', 'latex');
h_Dbar     = text(x_Dbar*1.07, y_Dbar-yt*0.12, '\={D}', 'FontSize', fs1,  'interpreter', 'latex');
h_Ebar     = text(x_Ebar, y_Ebar, '\={E}', 'FontSize', fs1,  'interpreter', 'latex');
h_Gbar     = text(x_Dbar*1.07, y_Dbar-yt*.35, '\={G}', 'FontSize', fs1, 'interpreter', 'latex');


slab = sprintf('$\\lambda = %.2f$, $\\alpha = %.2f$ , $x_{1r} = %.2f c$',  lambda, alpha, x1r/c); 

text(.38, 1.025, slab, 'Units', 'normalized', 'FontSize', fs1)
grid on

    hleg = legend([xcp, xll], {'Holdable Eq. Limits', 'Linear Region Boundary'},...
        'interpreter', 'latex', 'FontSize', fs2);
    hleg.Position = [0.7211 0.8082 0.2699 0.1391];

if saveon    
    print(gcf, '-dpdf', 'latex/figures/brake07.pdf', '-r900', '-opengl');     
end





