close all
clc
clear

addpath('functions')
figname = 'brake08.pdf';

figpath = fullfile(getfigpath(), figname);
saveon = 0;

F1 = figure(1);
width = 3;
height = 2;
set(F1,'color', 'w',...
    'defaulttextinterpreter', 'latex',...
    'PaperUnits','inches',...
    'PaperSize',[width height],...
    'PaperPosition',[0,0,width,height],...
    'Units','inches',...
    'Position',[1,1,width,height]);


gams = [0:0.01:1]';
alpha = .95;

% Normalized closed loop wn, \hat{wn}\w
Inq = 2*( 1 -gams).^2./( (1-gams).^2 + 4);
wnh_w = sqrt(1 + (1-Inq)./(alpha*Inq));


[AX, h1, h2] = plotyy(gams, Inq, gams, wnh_w);

h2.DisplayName = sprintf('$\\hat\\omega /\\omega$,  $\\alpha = %.2f$', alpha);
h1.DisplayName = 'Max $\lambda$';

h1.LineStyle = '-';
h2.LineStyle = '-.';

h3 = ylabel('Max $\lambda$', 'interpreter', 'latex', 'rotation', 90);
h3.FontSize = 12;

h5 = xlabel('$|\gamma|$', 'interpreter', 'latex');
h5.FontSize = 12;
set(AX(1), 'ytick', 0:.1:0.4)
set(AX(1), 'xtick', 0:0.25:1)

axes(AX(2));
set(AX(2), 'ytick', [0,20,40,80,120, 160])


h4 = ylabel('Min $\hat\omega/\omega$', 'interpreter', 'latex', 'rotation', 90);
h4.FontSize = 12;


set(AX,'Position',[.2 .27 .6 .7])

hleg = legend([h1, h2]);
hleg.Interpreter = 'latex';
hleg.FontSize = 10;
hleg.Position = [0.3211 0.7753 0.4520 0.1812];




if saveon
    saveEps(gcf, figpath); 
end



