% [fxk, t_k] = fxk_newton(w, zeta, bo, ubpm, alpha, xk)
%
% Evaluation of the non-zero damping switching curve.
%
% Applies newtons method to invert the x1*(t_k) for tk. Then evaluates
% x2*(tk) at tk to yield f_to(x1).

function [fxk, t_k] = fxk_newton(w, zeta, bo, ubpm, alpha, xk)

    wd = w*sqrt(1-zeta^2);
    sig = w*zeta;


    fx1 = @(t) ubpm*alpha*exp(-sig*t).*((-bo*zeta/wd/w)*sin(wd*t) - (bo/w^2)*cos(wd*t)) + ubpm*alpha*bo/w^2;
    fx2 = @(t) ubpm*alpha*exp(-sig*t).*((bo/wd)*sin(wd*t));
    
    gx1 = @(t, x1k) fx1(t) - x1k;

    dgx1 = fx2;

    % t should run between 0 and -pi/wd. Thus, our initial guess should 
    %  be in this range. If it is not, we can get a convergence to a point
    %  on the spiral OUTSIDE the region of interest. 
    t0 = -.5*pi/wd;
    t_k = t0;
    % xk = x0;



    for k = 1:30
        z = -gx1(t_k, xk)/dgx1(t_k);

        t_k = t_k + z;

    end

    gx1(t_k, xk);

    fxk = fx2(t_k);


end
