classdef ptosDamp_nom
    %PTOSDAMP_NOM Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        x_ell;
        lambda;
        Scurve;
        k2;
        k1;
        M;
        c;
        umax;
        alpha;
        ptosCurves;
        colors;
    end
    
    methods
        function obj = ptosDamp_nom(Scurve, lambda, c, umax, alpha)
            obj.lambda = lambda;
            obj.Scurve = Scurve;
            obj.c      = c;
            obj.umax   = umax;
            obj.alpha  = alpha;
            obj.x_ell  = c*umax*alpha*lambda;
            [ptosCurves, k1, k2] = makePTOS(obj);
            obj.ptosCurves = ptosCurves;
            obj.k1 = k1;
            obj.k2 = k2;
            
            colors.lower = 'r';
            colors.upper = 'r';
            colors.fp    = 'g';
            obj.colors = colors;
            
        end
        
        function F1 =  plot(obj, F1)
            if ~exist('F1', 'var')
                F1 = figure;
            end
            F1 = figure(F1);
            hold on;
            
            plot(obj.ptosCurves.x1_s, obj.ptosCurves.x2_ptos, obj.colors.fp, obj.ptosCurves.x1_s, obj.ptosCurves.x2_upper, obj.colors.upper);
            plot(obj.ptosCurves.x1_s, obj.ptosCurves.x2_lower, obj.colors.lower)
                
        end
        
        
        function F1 = plot_ref(obj, x1r, F1)
           if ~exist('F1', 'var')
                F1 = figure;
            end
            F1 = figure(F1);
            hold on;
            ubp = obj.umax - x1r/obj.c;
            ubm = -obj.umax - x1r/obj.c;
            
            sclFacs      = abs(sgnBeta(obj.ptosCurves.x1_s, ubm, ubp));
            x1s_scl      = obj.ptosCurves.x1_s.*sclFacs;
            fp_scl       = obj.ptosCurves.x2_ptos.*sclFacs;
            fp_scl_lower = fp_scl - ubp/obj.k2;
            fp_scl_upper = fp_scl - ubm/obj.k2;
            
            plot(x1s_scl, fp_scl, obj.colors.fp);
            plot(x1s_scl, fp_scl_lower, obj.colors.lower);
            plot(x1s_scl, fp_scl_upper, obj.colors.upper);
        end % end plot_ref
        
        
        
    end
    
end





function [ptosCurves, k1, k2] = makePTOS(obj)
    dx = obj.c/200;
    nn = length(obj.Scurve(:,1));
    x1_s = (linspace(obj.Scurve(1,1), obj.Scurve(end,1), nn))';
     Fxl = interp1(obj.Scurve(:,1), obj.Scurve(:,2), obj.x_ell, 'spline');

     Fxl_mdx = interp1(obj.Scurve(:,1), obj.Scurve(:,2), obj.x_ell-dx, 'spline');
     Fxl_pdx = interp1(obj.Scurve(:,1), obj.Scurve(:,2), obj.x_ell+dx, 'spline');
    % calc derivative
%     Df_xl = (Fxl-Fxl_dx)/(dx);
    Df_xl = (Fxl_pdx-Fxl_mdx)/(2*dx);
     
    k2 = abs(1/(Fxl - obj.x_ell*Df_xl));
    k1 = abs(Df_xl*k2);
     

     
     x2_ptos = zeros(nn, 1);
         for i=1:nn
            if( x1_s(i)<= -abs(obj.x_ell) )
                F_x = interp1(obj.Scurve(:,1), obj.Scurve(:,2), x1_s(i), 'spline');
                x2_ptos(i)= F_x - 1/k2;
            elseif( abs(x1_s(i)) < obj.x_ell )
                 x2_ptos(i)=-abs(k1/k2)*x1_s(i);
            elseif(x1_s(i) >= obj.x_ell)
                F_x = interp1(obj.Scurve(:,1), obj.Scurve(:,2), x1_s(i), 'spline');
                x2_ptos(i)= F_x + 1/k2;
            end
         end
    x2_upper = x2_ptos + 1/k2;    
    x2_lower = x2_ptos - 1/k2; 
    

    
  % produce output structure
    ptosCurves.x1_s        = x1_s;
    ptosCurves.x2_upper  = x2_upper;
    ptosCurves.x2_lower  = x2_lower;
    ptosCurves.x2_ptos   = x2_ptos;
    
end