% [ char_data ] = struct2meta(data)
%
% Use for generating a tagged data file to be read in latex with the
% package  catchfilebetweentags
%
% E.G. \usepackage{catchfilebetweentags}
%      
%   \begin{document}
%   \ExecuteMetaData[data/test.txt]{zeta}
%
%
% Inputs: data. This should be a struct.
%           The function is only defined for single level structs with
%           fields that are either strings or doubles.
%
% Outputs: char_data. is a string. This has the format:
%   %<*fieldname>
%   data.(fieldname)
%   %</fieldname>


function [ char_data ] = struct2meta(data)


fnames = fieldnames(data);


char_data = [];
for i = 1:length(fnames)
    
    s1 = [sprintf('%%<*%s>\n', fnames{i})];
    s3 = [sprintf('%%</%s>\n\n', fnames{i})];
    
    if isa(data.(fnames{i}), 'double')
        s2 = sprintf('%.3g\n', data.(fnames{i}));
        s2 = strrep(s2, 'e+', 'e');
        s2 = strrep(s2, 'e0', 'e');
    elseif isa(data.(fnames{i}), 'char')
        s2 = sprintf('%s\n', data.(fnames{i}));
    end
    
    char_data_i = [s1, s2, s3];
    
    
    char_data = [char_data, char_data_i];
    
    
end





end

